@@ -1,28 +1,38 @@
-/*
- * Copyright © 2002 Sun Microsystems, Inc., 4150 Network Circle, Santa Clara,
- * California 95054, U.S.A. All rights reserved.  Sun Microsystems, Inc. has
- * intellectual property rights relating to technology embodied in the product
- * that is described in this document. In particular, and without limitation,
- * these intellectual property rights may include one or more of the U.S.
- * patents listed at http://www.sun.com/patents and one or more additional
- * patents or pending patent applications in the U.S. and in other countries.
- * U.S. Government Rights - Commercial software. Government users are subject
- * to the Sun Microsystems, Inc. standard license agreement and applicable
- * provisions of the FAR and its supplements.  Use is subject to license terms.
- * Sun,  Sun Microsystems,  the Sun logo and  Java are trademarks or registered
- * trademarks of Sun Microsystems, Inc. in the U.S. and other countries.  This
- * product is covered and controlled by U.S. Export Control laws and may be
- * subject to the export or import laws in other countries.  Nuclear, missile,
- * chemical biological weapons or nuclear maritime end uses or end users,
- * whether direct or indirect, are strictly prohibited.  Export or reexport
- * to countries subject to U.S. embargo or to entities identified on U.S.
- * export exclusion lists, including, but not limited to, the denied persons
- * and specially designated nationals lists is strictly prohibited.
+/* Copyright (c) 2006, Sun Microsystems, Inc.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
+ *
+ *     * Redistributions of source code must retain the above copyright notice,
+ *       this list of conditions and the following disclaimer.
+ *     * Redistributions in binary form must reproduce the above copyright
+ *       notice, this list of conditions and the following disclaimer in the
+ *       documentation and/or other materials provided with the distribution.
+ *     * Neither the name of the Sun Microsystems, Inc. nor the names of its
+ *       contributors may be used to endorse or promote products derived from
+ *       this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
+ * THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 package org.javacc.parser;
 
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Enumeration;
+import java.util.Hashtable;
+import java.util.List;
+import java.util.Set;
 
 final class KindInfo
 {
@@ -61,119 +71,136 @@ public class RStringLiteral extends RegularExpression {
    */
   public String image;
 
-  static int maxStrKind = 0;
-  static int maxLen = 0;
-  static int charCnt = 0;
-  static Vector charPosKind = new Vector(); // Elements are hashtables
-                                            // with single char keys;
-  static int[] maxLenForActive = new int[100]; // 6400 tokens
+  public RStringLiteral() {
+  }
+
+  public RStringLiteral(Token t, String image) {
+    this.setLine(t.beginLine);
+    this.setColumn(t.beginColumn);
+    this.image = image;
+  }
+
+  private static int maxStrKind = 0;
+  private static int maxLen = 0;
+  private static int charCnt = 0;
+  private static List charPosKind = new ArrayList(); // Elements are hashtables
+                                                     // with single char keys;
+  private static int[] maxLenForActive = new int[100]; // 6400 tokens
   public static String[] allImages;
-  static int[][] intermediateKinds;
-  static int[][] intermediateMatchedPos;
+  private static int[][] intermediateKinds;
+  private static int[][] intermediateMatchedPos;
 
-  static int startStateCnt = 0;
-  static boolean subString[];
-  static boolean subStringAtPos[];
-  static Hashtable[] statesForPos;
+  private static int startStateCnt = 0;
+  private static boolean subString[];
+  private static boolean subStringAtPos[];
+  private static Hashtable[] statesForPos;
 
-  // Need to call this method after gnerating code for each lexical state. It
-  // initializes all the static variables, so that there is no interference
-  // between the various states of the lexer.
+  /**
+   * Initialize all the static variables, so that there is no interference
+   * between the various states of the lexer.
+   *
+   * Need to call this method after generating code for each lexical state.
+   */
   public static void ReInit()
   {
-     maxStrKind = 0;
-     maxLen = 0;
-     charPosKind = new Vector();
-     maxLenForActive = new int[100]; // 6400 tokens
-     intermediateKinds = null;
-     intermediateMatchedPos = null;
-     startStateCnt = 0;
-     subString = null;
-     subStringAtPos = null;
-     statesForPos = null;
+    maxStrKind = 0;
+    maxLen = 0;
+    charPosKind = new ArrayList();
+    maxLenForActive = new int[100]; // 6400 tokens
+    intermediateKinds = null;
+    intermediateMatchedPos = null;
+    startStateCnt = 0;
+    subString = null;
+    subStringAtPos = null;
+    statesForPos = null;
   }
 
   public static void DumpStrLiteralImages(java.io.PrintWriter ostr)
   {
-     String image;
-     int charCnt = 0, i;
+    String image;
+    int i;
+    charCnt = 0; // Set to zero in reInit() but just to be sure
 
-     ostr.println("public static final String[] jjstrLiteralImages = {");
+    ostr.println("");
+    ostr.println("/** Token literal values. */");
+    ostr.println("public static final String[] jjstrLiteralImages = {");
 
-     if (allImages == null || allImages.length == 0)
-     {
-        ostr.println("};");
-        return;
-     }
+    if (allImages == null || allImages.length == 0)
+    {
+      ostr.println("};");
+      return;
+    }
 
-     allImages[0] = "";
-     for (i = 0; i < allImages.length; i++)
-     {
-        if ((image = allImages[i]) == null ||
-            ((LexGen.toSkip[i / 64] & (1L << (i % 64))) == 0L &&
-             (LexGen.toMore[i / 64] & (1L << (i % 64))) == 0L &&
-             (LexGen.toToken[i / 64] & (1L << (i % 64))) == 0L) ||
-            (LexGen.toSkip[i / 64] & (1L << (i % 64))) != 0L ||
-            (LexGen.toMore[i / 64] & (1L << (i % 64))) != 0L ||
-            LexGen.canReachOnMore[LexGen.lexStates[i]] ||
-            ((Options.B("IGNORE_CASE") || LexGen.ignoreCase[i]) &&
-             (!image.equals(image.toLowerCase()) ||
-              !image.equals(image.toUpperCase()))))
-        {
-           allImages[i] = null;
-           if ((charCnt += 6) > 80)
-           {
-              ostr.println("");
-              charCnt = 0;
-           }
-
-           ostr.print("null, ");
-           continue;
-        }
-
-        String toPrint = "\"";
-
-        for (int j = 0; j < image.length(); j++)
-        {
-           if (image.charAt(j) <= 0xff)
-              toPrint += ("\\" + Integer.toOctalString((int)image.charAt(j)));
-           else
-           {
-              String hexVal = Integer.toHexString((int)image.charAt(j));
-
-              if (hexVal.length() == 3)
-                 hexVal = "0" + hexVal;
-              toPrint += ("\\u" + hexVal);
-           }
-        }
-
-        toPrint += ( "\", ");
-
-        if ((charCnt += toPrint.length()) >= 80)
-        {
-           ostr.println("");
-           charCnt = 0;
-        }
-
-        ostr.print(toPrint);
-     }
-
-     while (++i < LexGen.maxOrdinal)
-     {
+    allImages[0] = "";
+    for (i = 0; i < allImages.length; i++)
+    {
+      if ((image = allImages[i]) == null ||
+          ((LexGen.toSkip[i / 64] & (1L << (i % 64))) == 0L &&
+           (LexGen.toMore[i / 64] & (1L << (i % 64))) == 0L &&
+           (LexGen.toToken[i / 64] & (1L << (i % 64))) == 0L) ||
+          (LexGen.toSkip[i / 64] & (1L << (i % 64))) != 0L ||
+          (LexGen.toMore[i / 64] & (1L << (i % 64))) != 0L ||
+          LexGen.canReachOnMore[LexGen.lexStates[i]] ||
+          ((Options.getIgnoreCase() || LexGen.ignoreCase[i]) &&
+           (!image.equals(image.toLowerCase()) ||
+            !image.equals(image.toUpperCase()))))
+      {
+        allImages[i] = null;
         if ((charCnt += 6) > 80)
         {
-           ostr.println("");
-           charCnt = 0;
+          ostr.println("");
+          charCnt = 0;
         }
 
         ostr.print("null, ");
         continue;
-     }
+      }
 
-     ostr.println("};");
+      String toPrint = "\"";
+
+      for (int j = 0; j < image.length(); j++)
+      {
+        if (image.charAt(j) <= 0xff)
+          toPrint += ("\\" + Integer.toOctalString((int)image.charAt(j)));
+        else
+        {
+          String hexVal = Integer.toHexString((int)image.charAt(j));
+
+          if (hexVal.length() == 3)
+            hexVal = "0" + hexVal;
+          toPrint += ("\\u" + hexVal);
+        }
+      }
+
+      toPrint += ("\", ");
+
+      if ((charCnt += toPrint.length()) >= 80)
+      {
+        ostr.println("");
+        charCnt = 0;
+      }
+
+      ostr.print(toPrint);
+    }
+
+    while (++i < LexGen.maxOrdinal)
+    {
+      if ((charCnt += 6) > 80)
+      {
+        ostr.println("");
+        charCnt = 0;
+      }
+
+      ostr.print("null, ");
+      continue;
+    }
+
+    ostr.println("};");
   }
 
-  // Used for top level string literals
+  /**
+   * Used for top level string literals.
+   */
   public void GenerateDfa(java.io.PrintWriter ostr, int kind)
   {
      String s;
@@ -190,24 +217,25 @@ public class RStringLiteral extends RegularExpression {
      char c;
      for (int i = 0; i < len; i++)
      {
-        if (Options.B("IGNORE_CASE"))
+        if (Options.getIgnoreCase())
            s = ("" + (c = image.charAt(i))).toLowerCase();
         else
            s = "" + (c = image.charAt(i));
 
         if (!NfaState.unicodeWarningGiven && c > 0xff &&
-            !Options.B("JAVA_UNICODE_ESCAPE") &&
-            !Options.B("USER_CHAR_STREAM"))
+            !Options.getJavaUnicodeEscape() &&
+            !Options.getUserCharStream())
         {
            NfaState.unicodeWarningGiven = true;
            JavaCCErrors.warning(LexGen.curRE, "Non-ASCII characters used in regular expression." +
-              "Please make sure you use the correct Reader when you create the parser that can handle your character set.");
+              "Please make sure you use the correct Reader when you create the parser, " +
+              "one that can handle your character set.");
         }
 
         if (i >= charPosKind.size()) // Kludge, but OK
-           charPosKind.addElement(temp = new Hashtable());
+           charPosKind.add(temp = new Hashtable());
         else
-           temp = (Hashtable)charPosKind.elementAt(i);
+           temp = (Hashtable)charPosKind.get(i);
 
         if ((info = (KindInfo)temp.get(s)) == null)
            temp.put(s, info = new KindInfo(LexGen.maxOrdinal));
@@ -217,15 +245,15 @@ public class RStringLiteral extends RegularExpression {
         else
            info.InsertValidKind(ordinal);
 
-        if (!Options.B("IGNORE_CASE") && LexGen.ignoreCase[ordinal] &&
+        if (!Options.getIgnoreCase() && LexGen.ignoreCase[ordinal] &&
             c != Character.toLowerCase(c))
         {
            s = ("" + image.charAt(i)).toLowerCase();
 
            if (i >= charPosKind.size()) // Kludge, but OK
-              charPosKind.addElement(temp = new Hashtable());
+              charPosKind.add(temp = new Hashtable());
            else
-              temp = (Hashtable)charPosKind.elementAt(i);
+              temp = (Hashtable)charPosKind.get(i);
 
            if ((info = (KindInfo)temp.get(s)) == null)
               temp.put(s, info = new KindInfo(LexGen.maxOrdinal));
@@ -236,15 +264,15 @@ public class RStringLiteral extends RegularExpression {
               info.InsertValidKind(ordinal);
         }
 
-        if (!Options.B("IGNORE_CASE") && LexGen.ignoreCase[ordinal] &&
+        if (!Options.getIgnoreCase() && LexGen.ignoreCase[ordinal] &&
             c != Character.toUpperCase(c))
         {
            s = ("" + image.charAt(i)).toUpperCase();
 
            if (i >= charPosKind.size()) // Kludge, but OK
-              charPosKind.addElement(temp = new Hashtable());
+              charPosKind.add(temp = new Hashtable());
            else
-              temp = (Hashtable)charPosKind.elementAt(i);
+              temp = (Hashtable)charPosKind.get(i);
 
            if ((info = (KindInfo)temp.get(s)) == null)
               temp.put(s, info = new KindInfo(LexGen.maxOrdinal));
@@ -284,7 +312,7 @@ public class RStringLiteral extends RegularExpression {
         startState.charMoves = new char[1];
         startState.AddChar(image.charAt(i));
 
-        if (Options.B("IGNORE_CASE") || ignoreCase)
+        if (Options.getIgnoreCase() || ignoreCase)
         {
            startState.AddChar(Character.toLowerCase(image.charAt(i)));
            startState.AddChar(Character.toUpperCase(image.charAt(i)));
@@ -356,12 +384,12 @@ public class RStringLiteral extends RegularExpression {
 
   static int GetLine(int kind)
   {
-     return LexGen.rexprs[kind].line;
+     return LexGen.rexprs[kind].getLine();
   }
 
   static int GetColumn(int kind)
   {
-     return LexGen.rexprs[kind].column;
+     return LexGen.rexprs[kind].getColumn();
   }
 
   /**
@@ -417,7 +445,7 @@ public class RStringLiteral extends RegularExpression {
                  subStringAtPos[image.length() - 1] = true;
                  break;
               }
-              else if (Options.B("IGNORE_CASE") &&
+              else if (Options.getIgnoreCase() &&
                        StartsWithIgnoreCase((String)allImages[j], image))
               {
                  subString[i] = true;
@@ -431,24 +459,28 @@ public class RStringLiteral extends RegularExpression {
 
   static void DumpStartWithStates(java.io.PrintWriter ostr)
   {
-     ostr.println((Options.B("STATIC") ? "static " : "") + "private final int " +
+     ostr.println((Options.getStatic() ? "static " : "") + "private int " +
                   "jjStartNfaWithStates" + LexGen.lexStateSuffix + "(int pos, int kind, int state)");
      ostr.println("{");
      ostr.println("   jjmatchedKind = kind;");
      ostr.println("   jjmatchedPos = pos;");
 
-     if (Options.B("DEBUG_TOKEN_MANAGER"))
+     if (Options.getDebugTokenManager())
      {
         ostr.println("   debugStream.println(\"   No more string literal token matches are possible.\");");
-        ostr.println("   debugStream.println(\"   Currently matched the first \" + (jjmatchedPos + 1) + \" characters as a \" + tokenImage[jjmatchedKind] + \" token.\");");
+        ostr.println("   debugStream.println(\"   Currently matched the first \" " +
+                "+ (jjmatchedPos + 1) + \" characters as a \" + tokenImage[jjmatchedKind] + \" token.\");");
      }
 
      ostr.println("   try { curChar = input_stream.readChar(); }");
      ostr.println("   catch(java.io.IOException e) { return pos + 1; }");
 
-     if (Options.B("DEBUG_TOKEN_MANAGER"))
-        ostr.println("   debugStream.println(" + (LexGen.maxLexStates > 1 ? "\"<\" + lexStateNames[curLexState] + \">\" + " : "") + "\"Current character : \" + " +
-                 "TokenMgrError.addEscapes(String.valueOf(curChar)) + \" (\" + (int)curChar + \")\");");
+     if (Options.getDebugTokenManager())
+        ostr.println("   debugStream.println(" +
+                (LexGen.maxLexStates > 1 ? "\"<\" + lexStateNames[curLexState] + \">\" + " : "") +
+                "\"Current character : \" + " +
+                "TokenMgrError.addEscapes(String.valueOf(curChar)) + \" (\" + (int)curChar + \") " +
+                "at line \" + input_stream.getEndLine() + \" column \" + input_stream.getEndColumn());");
 
      ostr.println("   return jjMoveNfa" + LexGen.lexStateSuffix + "(state, pos + 1);");
      ostr.println("}");
@@ -457,16 +489,17 @@ public class RStringLiteral extends RegularExpression {
   private static boolean boilerPlateDumped = false;
   static void DumpBoilerPlate(java.io.PrintWriter ostr)
   {
-     ostr.println((Options.B("STATIC") ? "static " : "") + "private final int " +
+     ostr.println((Options.getStatic() ? "static " : "") + "private int " +
                   "jjStopAtPos(int pos, int kind)");
      ostr.println("{");
      ostr.println("   jjmatchedKind = kind;");
      ostr.println("   jjmatchedPos = pos;");
 
-     if (Options.B("DEBUG_TOKEN_MANAGER"))
+     if (Options.getDebugTokenManager())
      {
         ostr.println("   debugStream.println(\"   No more string literal token matches are possible.\");");
-        ostr.println("   debugStream.println(\"   Currently matched the first \" + (jjmatchedPos + 1) + \" characters as a \" + tokenImage[jjmatchedKind] + \" token.\");");
+        ostr.println("   debugStream.println(\"   Currently matched the first \" + (jjmatchedPos + 1) + " +
+                "\" characters as a \" + tokenImage[jjmatchedKind] + \" token.\");");
      }
 
      ostr.println("   return pos + 1;");
@@ -510,7 +543,7 @@ public class RStringLiteral extends RegularExpression {
 
      if (maxLen == 0)
      {
-        ostr.println((Options.B("STATIC") ? "static " : "") + "private final int " +
+        ostr.println((Options.getStatic() ? "static " : "") + "private int " +
                        "jjMoveStringLiteralDfa0" + LexGen.lexStateSuffix + "()");
 
         DumpNullStrLiterals(ostr);
@@ -523,20 +556,15 @@ public class RStringLiteral extends RegularExpression {
         boilerPlateDumped = true;
      }
 
-     if (!LexGen.mixed[LexGen.lexStateIndex] && NfaState.generatedStates != 0)
-        DumpStartWithStates(ostr);
-
-     boolean startNfaNeeded;
-     boolean returnLoopNeeded;
+     boolean createStartNfa = false;;
      for (i = 0; i < maxLen; i++)
      {
         boolean atLeastOne = false;
-        startNfaNeeded = false;
-        returnLoopNeeded = false;
-        tab = (Hashtable)charPosKind.elementAt(i);
+        boolean startNfaNeeded = false;
+        tab = (Hashtable)charPosKind.get(i);
         String[] keys = ReArrange(tab);
 
-        ostr.print((Options.B("STATIC") ? "static " : "") + "private final int " +
+        ostr.print((Options.getStatic() ? "static " : "") + "private int " +
                        "jjMoveStringLiteralDfa" + i + LexGen.lexStateSuffix + "(");
 
         if (i != 0)
@@ -618,20 +646,23 @@ public class RStringLiteral extends RegularExpression {
                     else
                        ostr.print("0L, ");
                  if (i <= maxLenForActive[j] + 1)
-                    ostr.println("old" + j + "); ");
+                    ostr.println("old" + j + ");");
                  else
                     ostr.println("0L);");
               }
               else if (NfaState.generatedStates != 0)
-                 ostr.println("      return jjMoveNfa" + LexGen.lexStateSuffix + "(" + NfaState.InitStateName() + ", " + (i - 1) + ");");
+                 ostr.println("      return jjMoveNfa" + LexGen.lexStateSuffix +
+                         "(" + NfaState.InitStateName() + ", " + (i - 1) + ");");
               else
                  ostr.println("      return " + i + ";");
            }
 
-           if (i != 0 && Options.B("DEBUG_TOKEN_MANAGER"))
+           if (i != 0 && Options.getDebugTokenManager())
            {
-              ostr.println("   if (jjmatchedKind != 0 && jjmatchedKind != 0x" + Integer.toHexString(Integer.MAX_VALUE) + ")");
-              ostr.println("      debugStream.println(\"   Currently matched the first \" + (jjmatchedPos + 1) + \" characters as a \" + tokenImage[jjmatchedKind] + \" token.\");");
+              ostr.println("   if (jjmatchedKind != 0 && jjmatchedKind != 0x" +
+                      Integer.toHexString(Integer.MAX_VALUE) + ")");
+              ostr.println("      debugStream.println(\"   Currently matched the first \" + " +
+                      "(jjmatchedPos + 1) + \" characters as a \" + tokenImage[jjmatchedKind] + \" token.\");");
 
               ostr.println("   debugStream.println(\"   Possible string literal matches : { \"");
 
@@ -639,7 +670,7 @@ public class RStringLiteral extends RegularExpression {
               {
                  if (i <= maxLenForActive[vecs])
                  {
-                    ostr.println(" + ");
+                    ostr.println(" +");
                     ostr.print("         jjKindsForBitVector(" + vecs + ", ");
                     ostr.print("active" + vecs + ") ");
                  }
@@ -666,24 +697,30 @@ public class RStringLiteral extends RegularExpression {
                  ostr.println("0L);");
 
 
-              if (i != 0 && Options.B("DEBUG_TOKEN_MANAGER"))
+              if (i != 0 && Options.getDebugTokenManager())
               {
-                 ostr.println("      if (jjmatchedKind != 0 && jjmatchedKind != 0x" + Integer.toHexString(Integer.MAX_VALUE) + ")");
-                 ostr.println("         debugStream.println(\"   Currently matched the first \" + (jjmatchedPos + 1) + \" characters as a \" + tokenImage[jjmatchedKind] + \" token.\");");
+                 ostr.println("      if (jjmatchedKind != 0 && jjmatchedKind != 0x" +
+                         Integer.toHexString(Integer.MAX_VALUE) + ")");
+                 ostr.println("         debugStream.println(\"   Currently matched the first \" + " +
+                         "(jjmatchedPos + 1) + \" characters as a \" + tokenImage[jjmatchedKind] + \" token.\");");
               }
               ostr.println("      return " + i + ";");
            }
            else if (NfaState.generatedStates != 0)
-              ostr.println("   return jjMoveNfa" + LexGen.lexStateSuffix + "(" + NfaState.InitStateName() + ", " + (i - 1) + ");");
+              ostr.println("   return jjMoveNfa" + LexGen.lexStateSuffix + "(" + NfaState.InitStateName() +
+                      ", " + (i - 1) + ");");
            else
               ostr.println("      return " + i + ";");
 
            ostr.println("   }");
         }
 
-        if (i != 0 && Options.B("DEBUG_TOKEN_MANAGER"))
-           ostr.println("   debugStream.println(" + (LexGen.maxLexStates > 1 ? "\"<\" + lexStateNames[curLexState] + \">\" + " : "") + "\"Current character : \" + " +
-                    "TokenMgrError.addEscapes(String.valueOf(curChar)) + \" (\" + (int)curChar + \")\");");
+        if (i != 0 && Options.getDebugTokenManager())
+           ostr.println("   debugStream.println(" +
+                   (LexGen.maxLexStates > 1 ? "\"<\" + lexStateNames[curLexState] + \">\" + " : "") +
+                   "\"Current character : \" + " +
+                   "TokenMgrError.addEscapes(String.valueOf(curChar)) + \" (\" + (int)curChar + \") " +
+                   "at line \" + input_stream.getEndLine() + \" column \" + input_stream.getEndColumn());");
 
         ostr.println("   switch(curChar)");
         ostr.println("   {");
@@ -694,13 +731,12 @@ public class RStringLiteral extends RegularExpression {
            key = keys[q];
            info = (KindInfo)tab.get(key);
            ifGenerated = false;
-           char c = (char)(key.charAt(0));
+           char c = key.charAt(0);
 
            if (i == 0 && c < 128 && info.finalKindCnt != 0 &&
                (NfaState.generatedStates == 0 || !NfaState.CanStartNfaUsingAscii(c)))
            {
               int kind;
-              Outer:
               for (j = 0; j < maxLongsReqd; j++)
                  if (info.finalKinds[j] != 0L)
                     break;
@@ -724,7 +760,7 @@ public class RStringLiteral extends RegularExpression {
                     {
                        LexGen.AddCharToSkip(c, kind);
 
-                       if (Options.B("IGNORE_CASE"))
+                       if (Options.getIgnoreCase())
                        {
                           if (c != Character.toUpperCase(c))
                              LexGen.AddCharToSkip(Character.toUpperCase(c), kind);
@@ -738,7 +774,7 @@ public class RStringLiteral extends RegularExpression {
            }
 
            // Since we know key is a single character ...
-           if (Options.B("IGNORE_CASE"))
+           if (Options.getIgnoreCase())
            {
               if (c != Character.toUpperCase(c))
                  ostr.println("      case " + (int)Character.toUpperCase(c) + ":");
@@ -815,6 +851,7 @@ public class RStringLiteral extends RegularExpression {
 
                        if (stateSetName != -1)
                        {
+                          createStartNfa = true;
                           ostr.println(prefix + "return jjStartNfaWithStates" +
                               LexGen.lexStateSuffix + "(" + i +
                               ", " + kindToPrint + ", " + stateSetName + ");");
@@ -915,7 +952,8 @@ public class RStringLiteral extends RegularExpression {
               {
 
                  if (NfaState.generatedStates != 0)
-                    ostr.println("         return jjMoveNfa" + LexGen.lexStateSuffix + "(" + NfaState.InitStateName() + ", 0);");
+                    ostr.println("         return jjMoveNfa" + LexGen.lexStateSuffix +
+                            "(" + NfaState.InitStateName() + ", 0);");
                  else
                     ostr.println("         return 1;");
               }
@@ -931,7 +969,7 @@ public class RStringLiteral extends RegularExpression {
            strings at this position. */
         ostr.println("      default :");
 
-        if (Options.B("DEBUG_TOKEN_MANAGER"))
+        if (Options.getDebugTokenManager())
            ostr.println("      debugStream.println(\"   No string literal matches possible.\");");
 
         if (NfaState.generatedStates != 0)
@@ -940,7 +978,8 @@ public class RStringLiteral extends RegularExpression {
            {
               /* This means no string literal is possible. Just move nfa with
                  this guy and return. */
-              ostr.println("         return jjMoveNfa" + LexGen.lexStateSuffix + "(" + NfaState.InitStateName() + ", 0);");
+              ostr.println("         return jjMoveNfa" + LexGen.lexStateSuffix +
+                      "(" + NfaState.InitStateName() + ", 0);");
            }
            else
            {
@@ -978,7 +1017,8 @@ public class RStringLiteral extends RegularExpression {
                  ostr.println("0L);");
            }
            else if (NfaState.generatedStates != 0)
-              ostr.println("   return jjMoveNfa" + LexGen.lexStateSuffix + "(" + NfaState.InitStateName() + ", " + i + ");");
+              ostr.println("   return jjMoveNfa" + LexGen.lexStateSuffix +
+                      "(" + NfaState.InitStateName() + ", " + i + ");");
            else
               ostr.println("   return " + (i + 1) + ";");
           }
@@ -986,6 +1026,9 @@ public class RStringLiteral extends RegularExpression {
 
         ostr.println("}");
      }
+
+     if (!LexGen.mixed[LexGen.lexStateIndex] && NfaState.generatedStates != 0 && createStartNfa)
+        DumpStartWithStates(ostr);
   }
 
   static final int GetStrKind(String str)
@@ -1012,8 +1055,8 @@ public class RStringLiteral extends RegularExpression {
      int i, j, kind, jjmatchedPos = 0;
      int maxKindsReqd = maxStrKind / 64 + 1;
      long[] actives;
-     Vector newStates = new Vector();
-     Vector oldStates = null, jjtmpStates;
+     List newStates = new ArrayList();
+     List oldStates = null, jjtmpStates;
 
      statesForPos = new Hashtable[maxLen];
      intermediateKinds = new int[maxStrKind + 1][];
@@ -1031,7 +1074,7 @@ public class RStringLiteral extends RegularExpression {
 
         try
         {
-           if ((oldStates = (Vector)initialState.epsilonMoves.clone()) == null ||
+           if ((oldStates = (List)initialState.epsilonMoves.clone()) == null ||
                oldStates.size() == 0)
            {
               DumpNfaStartStatesCode(statesForPos, ostr);
@@ -1059,7 +1102,7 @@ public class RStringLiteral extends RegularExpression {
            else
            {
               kind = NfaState.MoveFromSet(image.charAt(j), oldStates, newStates);
-              oldStates.removeAllElements();
+              oldStates.clear();
 
               if (j == 0 && kind != Integer.MAX_VALUE &&
                   LexGen.canMatchAnyChar[LexGen.lexStateIndex] != -1 &&
@@ -1097,21 +1140,21 @@ public class RStringLiteral extends RegularExpression {
               stateSets.put(stateSetString, stateSetString);
               for (p = 0; p < newStates.size(); p++)
               {
-                 if (seen[((NfaState)newStates.elementAt(p)).stateName])
-                    ((NfaState)newStates.elementAt(p)).inNextOf++;
+                 if (seen[((NfaState)newStates.get(p)).stateName])
+                    ((NfaState)newStates.get(p)).inNextOf++;
                  else
-                    seen[((NfaState)newStates.elementAt(p)).stateName] = true;
+                    seen[((NfaState)newStates.get(p)).stateName] = true;
               }
            }
            else
            {
               for (p = 0; p < newStates.size(); p++)
-                 seen[((NfaState)newStates.elementAt(p)).stateName] = true;
+                 seen[((NfaState)newStates.get(p)).stateName] = true;
            }
 
            jjtmpStates = oldStates;
            oldStates = newStates;
-           (newStates = jjtmpStates).removeAllElements();
+           (newStates = jjtmpStates).clear();
 
            if (statesForPos[j] == null)
               statesForPos[j] = new Hashtable();
@@ -1135,21 +1178,21 @@ public class RStringLiteral extends RegularExpression {
   static void DumpNfaStartStatesCode(Hashtable[] statesForPos,
                                               java.io.PrintWriter ostr)
   {
+      if (maxStrKind == 0) { // No need to generate this function
+         return;
+      }
+
      int i, maxKindsReqd = maxStrKind / 64 + 1;
      boolean condGenerated = false;
      int ind = 0;
 
-     if (maxStrKind == 0) // No need to generate this function
-        return;
-
-
-     ostr.print("private" + (Options.B("STATIC") ? " static" : "") + " final int jjStopStringLiteralDfa" +
+     ostr.print("private" + (Options.getStatic() ? " static" : "") + " final int jjStopStringLiteralDfa" +
                   LexGen.lexStateSuffix + "(int pos, ");
      for (i = 0; i < maxKindsReqd - 1; i++)
         ostr.print("long active" + i + ", ");
      ostr.println("long active" + i + ")\n{");
 
-     if (Options.B("DEBUG_TOKEN_MANAGER"))
+     if (Options.getDebugTokenManager())
         ostr.println("      debugStream.println(\"   No more string literal token matches are possible.\");");
 
      ostr.println("   switch (pos)\n   {");
@@ -1261,7 +1304,7 @@ public class RStringLiteral extends RegularExpression {
      ostr.println("   }");
      ostr.println("}");
 
-     ostr.print("private" + (Options.B("STATIC") ? " static" : "") + " final int jjStartNfa" +
+     ostr.print("private" + (Options.getStatic() ? " static" : "") + " final int jjStartNfa" +
                   LexGen.lexStateSuffix + "(int pos, ");
      for (i = 0; i < maxKindsReqd - 1; i++)
         ostr.print("long active" + i + ", ");
@@ -1270,7 +1313,8 @@ public class RStringLiteral extends RegularExpression {
      if (LexGen.mixed[LexGen.lexStateIndex])
      {
          if (NfaState.generatedStates != 0)
-            ostr.println("   return jjMoveNfa" + LexGen.lexStateSuffix + "(" + NfaState.InitStateName() + ", pos + 1);");
+            ostr.println("   return jjMoveNfa" + LexGen.lexStateSuffix +
+                    "(" + NfaState.InitStateName() + ", pos + 1);");
          else
             ostr.println("   return pos + 1;");
 
@@ -1286,22 +1330,24 @@ public class RStringLiteral extends RegularExpression {
      ostr.println(", pos + 1);");
      ostr.println("}");
   }
+  /**
+   * Return to original state.
+   */
+  public static void reInit()
+  {
+    ReInit();
 
-   public static void reInit()
-   {
-      maxStrKind = 0;
-      maxLen = 0;
-      charCnt = 0;
-      charPosKind = new Vector();
-      maxLenForActive = new int[100];
-      allImages = null;
-      intermediateKinds = null;
-      intermediateMatchedPos = null;
-      startStateCnt = 0;
-      subString = null;
-      subStringAtPos = null;
-      statesForPos = null;
-      boilerPlateDumped = false;
-   }
+    charCnt = 0;
+    allImages = null;
+    boilerPlateDumped = false;
+  }
 
+  public StringBuffer dump(int indent, Set alreadyDumped) {
+    StringBuffer sb = super.dump(indent, alreadyDumped).append(' ').append(image);
+    return sb;
+  }
+
+  public String toString() {
+    return super.toString() + " - " + image;
+  }
 }
